/*
 * A2V : A2C_Convert.cpp
 *
 * Copyright 1998, Sony Computer Entertainment Inc.
 */

#include "stdafx.h"
#include "A2VDlg.h"

#include "encvag.h"

void printChunkAIFFHead( ChunkAIFFHeader *aChunk );
void readChunkHead( ChunkHeader *aChunk );
void readChunkAIFFHead( ChunkAIFFHeader *aChunk );
float sane2ld( unsigned char *sane );


void CA2VDlg::Convert( short loopon )
{
	ChunkAIFFHeader aChunkAIFF;
	ChunkHeader aChunk;
	CommonChunk aCommonRv, aCommon;
	Marker aMarkRv, aMark[2];
	SoundDataChunk aSndDataChunk;
	CWaitCursor  waitCursor;

	long total, ssnd_pos;
	short i, skip, marks=0, loopflg;	/* 0: no loop, 1: loop */
	float sampleRate = (float)44100;
	char name[256], c;
	size_t fcount;
	VAGheader aVh;

	errStr = "";
	
	total = 12;
	try{
		readChunkAIFFHead( arFp, &aChunkAIFF );	/* FORM XXXX AIFF */
	}
	catch(...){
		errStr = "Err in reading AIFF head chunk";
		goto ABORT;
	}

	if( aChunkAIFF.ckType != AIFF ) {
		errStr = "Sorry, A2V_Win only supports NOT compressed AIFF";
		goto ABORT;
	}

	while( (total < aChunkAIFF.ckSize) && (total > 0) ) 
	{
		try{
			readChunkHead( arFp, &aChunk );
		}
		catch(...){
			errStr = "Error in reading a chunk head.";
			goto ABORT;
		}
		total += aChunk.ckSize + sizeof( ChunkHeader );
		skip=0;

		switch( aChunk.ckID ) {
			case COMM: /* ===================== C O M M ===================== */
				try{
					arFp.Read( &aCommonRv.numChannels, sizeof( short ) );
					arFp.Read( &aCommonRv.numSampleFrames, sizeof( unsigned long ) );
					arFp.Read( &aCommonRv.sampleSize, sizeof( short ) );
					arFp.Read( &aCommonRv.sane, sizeof( char ) * 10 );
				}
				catch(...){
					errStr = "Error in rading AIFF Common chunk.";
					goto ABORT;
				}
				aCommon.numChannels = sEndian( aCommonRv.numChannels );
				aCommon.numSampleFrames = lEndian( aCommonRv.numSampleFrames );
				aCommon.sampleSize = sEndian( aCommonRv.sampleSize );
				sampleRate = sane2ld( aCommonRv.sane );
				if( aCommon.numChannels > 1 ) {
					errStr = "Sorry, AIFF must be monoral.";
					goto ABORT;
				}
				if( aCommon.sampleSize != 16 ) {
					errStr = " Sorry, AIFF must be 16 bit PCM.";
					goto ABORT;
				}
				skip = 1;
				break;
			case APPL: /* ===================== A P P L ===================== */
				break;
			case INST: /* ===================== I N S T ===================== */
				break;
			case SSND: /* ===================== S S N D ===================== */
				try{
					arFp.Read( &aSndDataChunk, sizeof( SoundDataChunk ) );
				}
				catch(...){
					errStr = "Error in in rading AIFF sound data chunk.";
					goto ABORT;
				}
				ssnd_pos = arFp.GetPosition();
				arFp.Seek( (aCommon.numSampleFrames * 2), CFile::current );
				
				skip = 1;
				break;
			case MARK: /* ===================== M A R K ===================== */
				try{
					arFp.Read( &marks, sizeof( short ) );
					marks = sEndian( marks );

					for( i = 0; i < marks; i++ ) {
						arFp.Read( &aMarkRv.id, sizeof( short ) );
						arFp.Read( &aMarkRv.position, sizeof( long ) );
						arFp.Read( &aMarkRv.size, sizeof( char ) );

						fcount = 0xff & aMarkRv.size;
						arFp.Read( name, sizeof( char ) * fcount);
						if( (aMarkRv.size % 2) == 0 ) { /* for padding */
							arFp.Read( &c, sizeof( char ) );
						}						
						name[aMarkRv.size] = 0;
						if( i < 2 ) {
							aMark[i].id = sEndian( aMarkRv.id );
							aMark[i].position = lEndian( aMarkRv.position );
							aMark[i].size = aMarkRv.size;

						}
					}					
					if( marks > 2 ){
						errStr.Format( "%d marks detected, but first two will be used for the looping.",marks );
						AfxMessageBox( errStr );
					}
					skip = 1;
				}
				catch(...){
					errStr = "Error in rading AIFF Common chunk.";
					goto ABORT;
				}			

				break;
			default: /* ===================== default ===================== */
				break;
		}
		if( skip == 0 ) {
			arFp.Seek( total, CFile::begin );
		}
	}
	loopflg = 0;
	if( marks > 0 ) {
		loopflg = 1;
	} 
	else if( loopon == 1 ) {
		loopflg = 1;
		aMark[0].position = 0;
		aMark[1].position = aCommon.numSampleFrames;
	}


	if( loopon == 2 ) {
		loopflg = 0;
	}

	vagFile.Write( &aVh, sizeof( VAGheader ) );
	
	arFp.Seek( ssnd_pos, CFile::begin );

	try{
		Encode( arFp, aCommon.numSampleFrames, loopflg, aMark[0].position, aMark[1].position );
	}
	catch(CString){
		goto ABORT;
	}

	/* write VAG header */
	aVh.format = lEndian( 0x56414770 ); /* "VAGp" */
	aVh.ver = lEndian( 0x20L );
	aVh.ssa = 0;
	aVh.size = lEndian( vagFile.GetPosition() - sizeof( VAGheader ));
	aVh.fs = lEndian( (long)sampleRate );
	aVh.volL = aVh.volR = aVh.pitch = aVh.ADSR1 = aVh.ADSR2 = aVh.reserved = 0;
	for( i = 0; i < 16; i++ ) {
		aVh.name[i] = 0;
	}
	sprintf( aVh.name, "%s", noExName );
	vagFile.Seek( 0, CFile::begin );

	vagFile.Write( &aVh, sizeof( VAGheader ) );

	return;

ABORT:
	AfxMessageBox(errStr);
	return;
}


void CA2VDlg::readChunkHead( CFile& arFp, ChunkHeader *aChunk )
{
	ChunkHeader tmp;

	arFp.Read( &tmp, sizeof( ChunkHeader ) );
	aChunk->ckID   = lEndian( tmp.ckID );
	aChunk->ckSize = lEndian( tmp.ckSize );
}

void CA2VDlg::readChunkAIFFHead( CFile& arFp, ChunkAIFFHeader *aChunk )
{
	ChunkAIFFHeader tmp;

	arFp.Read( &tmp, sizeof( ChunkAIFFHeader ) );
	aChunk->ckID   = lEndian( tmp.ckID );
	aChunk->ckSize = lEndian( tmp.ckSize );
	aChunk->ckType = lEndian( tmp.ckType );
}

float sane2ld( unsigned char *sane )
{
	float tmp;
	float massive;
	short exponent;
	short i;
	short dat;
	unsigned char c2, c3;

	exponent = ( (0x3f & sane[0]) << 8 ) | sane[1] + 1;

	massive = 0.0f;
	c2 = sane[2];
	c3 = sane[3];
	dat = c2 << 8 | c3;
	for( i = 0; i < 16; i++ ) {
		if( (1 & (dat >> (15-i))) != 0 ) {
			massive += ((float)1.0/(1L<<i));
		}
	}
	tmp = massive * (1L << exponent);
	return( tmp );
}

