#include "stdafx.h"
#include "vagPrev.h"
#include "InfoDlg.h"

#include "vagPrevDoc.h"
#include "vagPrevView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MALLOC_MAX 1
char spu_malloc_rec [SPU_MALLOC_RECSIZ * (MALLOC_MAX + 1)];
#define TRANSFERRED_BY_IO 
#define S_PITCH 0x1000
#define PITCH_INITIAL 60
#define PITCH_MAX 108
#define PITCH_MIN 24
#define	REVERB_DEPTH 	0x1400
#define REVERB_MODE		SPU_REV_MODE_HALL


#define WORK_AROWND

/////////////////////////////////////////////////////////////////////////////
// CVagPrevView

IMPLEMENT_DYNCREATE(CVagPrevView, CFormView)

BEGIN_MESSAGE_MAP(CVagPrevView, CFormView)
	//{{AFX_MSG_MAP(CVagPrevView)
	ON_BN_CLICKED(IDC_KeyOn, OnKeyOn)
	ON_BN_CLICKED(IDC_KeyOff, OnKeyOff)
	ON_WM_HSCROLL()
	ON_BN_CLICKED(IDC_BtnSet, OnBtnSet)
	ON_BN_CLICKED(IDC_Inform, OnInform)
	ON_BN_CLICKED(IDC_CHECK_REV, OnCheckRev)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVagPrevView Conuctor/Destructor

CVagPrevView::CVagPrevView()
	: CFormView(CVagPrevView::IDD)
{
	isLoaded = 0;

	initializeSpu();
	initializeRev();	
}

CVagPrevView::~CVagPrevView()
{
	SpuSetKey (SpuOff, SPU_ALLCH);
	SpuQuit ();
}

void CVagPrevView::OnInitialUpdate()
{
	CView::OnInitialUpdate();

	if( isLoaded == 0 )
	{
		m_editPitch = PITCH_INITIAL;
		m_combADSR = "cut";
		m_checkRev = FALSE;
		UpdateData(FALSE);
		m_btnKeyOn.EnableWindow(FALSE);
		m_btnKeyOff.EnableWindow(FALSE);
		m_btnInform.EnableWindow(FALSE);
		m_sldPitch.SetRangeMax( PITCH_MAX );
		m_sldPitch.SetRangeMin( PITCH_MIN );
		m_sldPitch.SetPos( PITCH_INITIAL );
		m_sldPitch.SetPageSize( 1 );
		UpdateData(TRUE);
	}
}


void CVagPrevView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CVagPrevView)
	DDX_Control(pDX, IDC_Inform, m_btnInform);
	DDX_Control(pDX, IDC_SldPitch, m_sldPitch);
	DDX_Control(pDX, IDC_KeyOff, m_btnKeyOff);
	DDX_Control(pDX, IDC_KeyOn, m_btnKeyOn);
	DDX_Text(pDX, IDC_EditPitch, m_editPitch);
	DDV_MinMaxUInt(pDX, m_editPitch, 24, 108);
	DDX_CBString(pDX, IDC_CombADSR, m_combADSR);
	DDX_Check(pDX, IDC_CHECK_REV, m_checkRev);
	//}}AFX_DATA_MAP
}

BOOL CVagPrevView::PreCreateWindow(CREATESTRUCT& cs)
{	
	return CFormView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CVagPrevView Diagnostics

#ifdef _DEBUG
void CVagPrevView::AssertValid() const
{
	CFormView::AssertValid();
}

void CVagPrevView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CVagPrevDoc* CVagPrevView::GetDocument()
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CVagPrevDoc)));
	return (CVagPrevDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CVagPrevView : message handler

void CVagPrevView::OnKeyOn() 
{
	UpdateData(TRUE);

	setNoteAttr();

	setNote( m_editPitch );

	SpuSetKey(SPU_ON, SPU_00CH );
}


void CVagPrevView::OnKeyOff() 
{
	SpuSetKey (SPU_OFF, SPU_00CH);
}


void CVagPrevView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	UpdateData(FALSE);
	m_editPitch = m_sldPitch.GetPos();
	
	setNote( m_editPitch );

	CFormView::OnHScroll(nSBCode, nPos, pScrollBar);
}


void CVagPrevView::OnBtnSet() 
{
	UpdateData(TRUE);
	m_sldPitch.SetPos( m_editPitch );
}


void CVagPrevView::OnInform() 
{
	CInfoDlg dlg;
	CString str = "";
	char text[255];

	wsprintf(text, "sampling freq.: %5d Hz       \n", m_freq*10 );
	str += text;

	switch ( SpuGetTransferMode () ) 
	{
        case SpuTransferByDMA:
             str += "\nTransfer mode: DMA \n"; 
	         break;
        case SpuTransferByIO:
             str += "\nTransfer mode: IO \n"; 
	         break;
        default:
             str += "\nError: Transfer mode unknown. \n"; 
	    break;
    }

	wsprintf(text,"Transfer start addr: 0x%x \n", SpuGetTransferStartAddr () );
	str	+= text;

    SpuGetVoiceAttr (&s_attr);
    wsprintf( text,"\nvolumex.left: 0x%04x,   volumex.right: 0x%04x \n", 
              s_attr.volumex.left ,
              s_attr.volumex.right
			);
	str	+= text;

    wsprintf( text,  "volume.left:  0x%04x,   volume.right:  0x%04x \n", 
              s_attr.volume.left ,
              s_attr.volume.right
			);
	str	+= text;

    wsprintf( text,"\npitch:  0x%x \naddr:  0x%x \n", 
              s_attr.pitch,  
              s_attr.addr    
			);
	str	+= text;

    wsprintf( text,"\nADSR rate: \n       a: 0x%x,  d: 0x%x,  s: 0x%x,  r: 0x%x\n       sustain level: 0x%x\n", 
              s_attr.ar,  
              s_attr.dr,
              s_attr.sr,
              s_attr.rr,
              s_attr.sl
			);
	str	+= text;
 
	wsprintf( text,"ADSR mode: \n       a: 0x%x,  s: 0x%x,  r: 0x%x \n", 
              s_attr.a_mode,  
              s_attr.s_mode,
              s_attr.r_mode
			);
	str	+= text;
	
	dlg.m_txtInfo = str;
	dlg.DoModal();
}


void CVagPrevView::OnCheckRev() 
{
	UpdateData(TRUE);
	if( m_checkRev == TRUE )
	{
		SpuSetReverbModeDepth( REVERB_DEPTH, REVERB_DEPTH );
	}
	else
	{
		SpuSetReverbModeDepth( 0, 0 );
	}
}


/////////////////////////////////////////////////////////////////////////////
// CVagPrevView : user function

void CVagPrevView::initializeSpu()
{
    SpuCommonAttr c_attr;
	SpuReverbAttr r_attr; 
	SpuVolume  vol;

	SpuInit ();
	SpuInitMalloc (MALLOC_MAX, spu_malloc_rec);

#if 1
	//-- nise prevention
	SpuSetReverb( SPU_OFF );
	vol.left = 0;
	vol.right = 0;
	r_attr.depth = vol;
	SpuSetReverbDepth( &r_attr );
#endif
//	SpuClearReverbWorkArea( SPU_REV_MODE_HALL );

	// common attribute setting
	c_attr.mask = (SPU_COMMON_MVOLL |  SPU_COMMON_MVOLR);
	c_attr.mvol.left  = 0x3fff;	/* Master volume (left) */
	c_attr.mvol.right = 0x3fff;	/* Master volume (right) */
	SpuSetCommonAttr (&c_attr);

	
    // items of voice attribute
    s_attr.mask = ( SPU_VOICE_VOLL |
		            SPU_VOICE_VOLR |
                    SPU_VOICE_VOLMODEL|
                    SPU_VOICE_VOLMODER
		          );

    // set the attributes with all voice
    s_attr.voice = SPU_ALLCH;

    // value of each voice attribute
    s_attr.volume.left  = 0x3fff;		         /* Left volume */
    s_attr.volume.right = 0x3fff;		         /* Right volume */
	s_attr.volmode.left =  SPU_VOICE_DIRECT;
	s_attr.volmode.right =  SPU_VOICE_DIRECT;

#ifndef WORK_AROWND
	SpuSetVoiceAttr (&s_attr);
#else
	short i;
	for( i = 0; i < 24; i++ )
	{
		Sleep(5);
		SpuNSetVoiceAttr (i, &s_attr);
	}	
#endif

}


void CVagPrevView::initializeRev()
{
	SpuReverbAttr r_attr;

	r_attr.mask = ( SPU_REV_DEPTHL | SPU_REV_DEPTHR | SPU_REV_MODE );
	r_attr.depth.left = 0;
	r_attr.depth.right = 0;
	r_attr.mode = SPU_REV_MODE_HALL;
//   r_attr.mode = (SPU_REV_MODE_CLEAR_WA | SPU_REV_MODE_OFF);
	SpuSetReverbModeParam( &r_attr );
	SpuSetReverb( SPU_ON );
	SpuSetReverbModeDepth( 0, 0 );
	SpuSetReverbVoice( SPU_ON, SPU_00CH );
}


void CVagPrevView::loadVag( long vagSize, unsigned char* vagData )
{
	unsigned long addr;
	static unsigned long a_addr;
    unsigned long size;

	if( isLoaded != 0 )
	{
		SpuFree( a_addr );
	}

	m_btnKeyOn.EnableWindow(FALSE);

	// wave transfer 
    SpuSetTransferMode (SpuTransByIO); /* transfer by I/O */
    a_addr = SpuMalloc (vagSize);
	addr = SpuSetTransferStartAddr (a_addr);
 	size = SpuWrite (vagData+48, vagSize);
 
	// attribute setting
    s_attr.mask = ( SPU_VOICE_WDSA | SPU_VOICE_SAMPLE_NOTE );
    s_attr.addr         = a_addr;  /* Waveform data start address */
	s_attr.sample_note  = ( (60 + (44100/m_freq -1)*12) << 8 );

	SpuSetVoiceAttr (&s_attr);

	isLoaded = 1;
	m_btnKeyOn.EnableWindow(TRUE);
	m_btnKeyOff.EnableWindow(TRUE);
	m_btnInform.EnableWindow(TRUE);
	m_btnKeyOn.SetFocus();
}



void CVagPrevView::setNoteAttr() 
{

	s_attr.mask = ( SPU_VOICE_ADSR_AR |
		            SPU_VOICE_ADSR_DR |
		            SPU_VOICE_ADSR_SR |
		            SPU_VOICE_ADSR_RR |
		            SPU_VOICE_ADSR_SL |
		            SPU_VOICE_ADSR_AMODE |
		            SPU_VOICE_ADSR_SMODE |
		            SPU_VOICE_ADSR_RMODE
		          );

	s_attr.a_mode       = SPU_VOICE_EXPIncN;	 /* Attack curve */
    s_attr.s_mode       = SPU_VOICE_EXPDecN;	 /* Sustain curve */
    s_attr.r_mode       = SPU_VOICE_EXPDecN;	 /* Release curve */

	if( m_combADSR == "cut" )
	{
		s_attr.ar           = 0;			         /* Attack rate value */
		s_attr.dr           = 15;			         /* Decay rate value */
		s_attr.sl           = 15;			         /* Sustain level value */
	    s_attr.sr           = 127;			         /* Sustain rate value */
		s_attr.rr           = 0;		             /* Release rate value */
	}
	else if( m_combADSR == "type1" )
	{
		s_attr.ar           = 30;			         /* Attack rate value */
		s_attr.dr           = 14;			         /* Decay rate value */
		s_attr.sl           = 14;			         /* Sustain level value */
	    s_attr.sr           = 52;			         /* Sustain rate value */
		s_attr.rr           = 13;		             /* Release rate value */
	}

	else if( m_combADSR == "type2" )
	{
		s_attr.ar           = 50;			         /* Attack rate value */
		s_attr.dr           = 14;			         /* Decay rate value */
		s_attr.sl           = 14;			         /* Sustain level value */
	    s_attr.sr           = 115;			         /* Sustain rate value */
		s_attr.rr           = 14;		             /* Release rate value */
	}

	SpuSetVoiceAttr (&s_attr);
}


void CVagPrevView::setNote(unsigned short note)
{
    s_attr.mask = ( SPU_VOICE_NOTE );

	s_attr.note = note<<8;

	SpuSetVoiceAttr (&s_attr);
}


void CVagPrevView::OnButton2() 
{	
	long err;
	CString str = "";

	err = SpuClearReverbWorkArea( SPU_REV_MODE_HALL );

	str += "return = " + err;
	AfxMessageBox(str);

}
